/***************************************************************************
 *            Comm.c
 *
 *  Wed Nov  2 17:06:55 2005
 *  Copyright  2005  Oliver Schwaneberg, FB02.FH-Bonn-Rhein-Sieg, BGIA
 *  oliver.schwaneberg@smail.inf.fh-brs.de
 ****************************************************************************/
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/delay.h>
#include <avr/pgmspace.h>
#include <compat/deprecated.h>

#include <inttypes.h>
#include <stdlib.h>
#include <stdio.h>

#define TRUE 1
#define FALSE 0

static char comLOCK = FALSE;

char isCharAvailable(void)
{	
	// Does the RX0 bit of the USART Status and Control Register
	// indicate a char has been received?
	if ( (UCSRA & (0x80)) ) return 1;
	else return 0;
}

char receiveChar(void)
{
	// Return the char in the UDR0 register
	return UDR;
}

void sendChar(char data)
{
	if(comLOCK)
		return;
	comLOCK = TRUE;
	
    uint16_t i = 0;

    // To send data with the USART put the data in the USART data register 
    UDR = data;

    // Check to see if the global interrupts are enabled
    if(SREG & 0x80)
    {
	// Wait until the byte is sent or we count out
		while ( !(UCSRA&0x40) && (i<40000) ) //Das war ne 10000, aber wir haben ja 16MHz statt 2 MHz
		{
			asm volatile ("nop"::); //Hier muss quasi der Takt halbiert werden, da 80000 nicht in uint16 passt.
			i++;
		}
    }
    else  // Wait until the byte is sent
        while( !(UCSRA&0x40) );
        
    	// Clear the TXCflag        
	UCSRA=UCSRA|0x40; 

	comLOCK = FALSE;		
}

void sendString(char s[])
{	
	int i = 0;
	
	while(i < 64) // don't get stuck if it is a bad string
	{
		if( s[i] == '\0' ) break; // quit on string terminator
		sendChar(s[i++]);
	}
}


void sendInteger(int ADCresult)
{
	char *intpart= (char*) malloc(sizeof(char)*10);
	
	itoa(ADCresult, intpart, 10);
	
	sendString(intpart);
	
	free(intpart);
}

void sendUnsignedInteger(int ADCresult)
{
	char *intpart= (char*) malloc(sizeof(char)*10);
	
	utoa(ADCresult, intpart, 10);
	
	sendString(intpart);
	
	free(intpart);
}

void sendFloat(float ADCresult){
	char *intpart= (char*) malloc(sizeof(char)*10);
	
	dtostrf(ADCresult, 2, 3, intpart);
	
	sendString(intpart);
	
	free(intpart);
}

void USARTinit(void)
{
	/*
	 * Nenene, son Spökes machen wir hier nicht!
	 * Die CPU bleibt mal schön auf 8MHz getaktet!
	 * (Im Originalcode wird die CPU nur auf 2MHz hochgefahren)
	 */
    cli();
    // Set the USART baudrate registers for 19200
	int UBRR = 51;//103; //19200 Baud
    UBRRH = (uint8_t) (UBRR>>8);
    UBRRL = (uint8_t) (UBRR);

    // Enable 2x speed change
    //UCSRA = (1<<U2X0);
	//UBRR = 103; //19200 Baud
	//UBRR = 51; //38400 Baud
	
    // Enable receiver and transmitter
    UCSRB = (1<<RXEN)|(1<<TXEN)|(0<<RXCIE)|(0<<UDRIE);

    // Set the USART to asynchronous at 8 bits no parity and 1 stop bit
	UCSRC = (1 << URSEL) | (1 << UCSZ1) | (1 << UCSZ0);
    //UCSRC = (0<<UMSEL)|(0<<UPM0)|(0<<USBS)|(3<<UCSZ0)|(0<<UCPOL);
        
    // Enable interrupts
    sei(); 
    
    // Setup the interrupt mask and flags
	//### Das folgende Statement sollte in die IO-Init-Methode umziehen ####
	//PCMSK1 = (0<<PINB6) | (0<<PINB4) | (1<<PINB7);       // set pin-change interrupt mask
	//#### Für die Codeverschönerung ####
  	//EIFR   = (1<<PCIF1);                    // clear external intterupt flag 1
	//EIMSK  = (1<<PCIE1);                // enable external interrupt 1    
}
