/***************************************************************************
 *  Prometeus.c
 *  Copyright  2007-2008 Gecko & Olli S.
 *  oliver.schwaneberg@smail.inf.fh-brs.de, gutergecko@gmx.de
 *
 *  Teilweise basierend auf den Codebeispielen von ATMEL Norway
 *  und der Skinner Software.
 *
 ***************************************************************************

 ============================ CHANGELOG ====================================
	Version 1.0		Der Urknall
	Version 1.1		Hardware-PWM für Weißkanal
	Version 1.2		10Bit Hardware PWM für Weißkanal
	Version 1.3beta	Intelligenteres Timing zur Verbesserung des Software-PWM
	Version 1.3		Behobene Absturzursache: Raceconditions im Menu
	* BREAK *
	Version 1.0-LMC Neue Version für neue Hardware: den LeuchtmuffController
	Version 1.01-LMC Mini-Bugfix an Powermanagement und DCF-Fehlerkorrektur
 ===========================================================================

 ######################### CURRENT ISSUES ##################################
 * Neue Menustruktur			( )
 ###########################################################################*/

#include <avr/io.h>
#include <avr/delay.h>
#include <avr/interrupt.h>
#include <compat/deprecated.h>
#include <avr/pgmspace.h>
#include <avr/iom32.h>
#include <avr/sleep.h>
#include <inttypes.h>

#include <stdlib.h>

#include "Prometheus.h"

#define TRUE 1
#define FALSE 0

char rxChar;
//static uint8_t roundrobin = 0; Brauchen wir nicht mehr
static uint8_t calculateFlag = FALSE; //Wenn TRUE wird "calculateIntensities" ausgeführt.


#define RECEIVER //UART-Command-Parser

static volatile uint8_t LOCK = 0;
static volatile uint8_t LOCK2 = 0;

//Aktuelle Intensität
static volatile uint16_t intensityRed = 1;
static volatile uint16_t intensityGreen = 1;
static volatile uint16_t intensityBlue = 1;
static volatile uint16_t intensityWhite = 1;

//Aktuelle Zeit (Millisecunde des Tages)
static volatile uint32_t timeMs = 0;

//Intervalzeit
static volatile uint8_t intervalTime = 0;

//Das ist der 40 Byte große Cache für die aktuell relevanten Einträge
static volatile uint32_t lastEntryRedTime = 0;
static volatile uint32_t nextEntryRedTime = 0;
static volatile uint16_t  lastEntryRedIntensity = 0;
static volatile uint16_t  nextEntryRedIntensity = 0;

static volatile uint32_t lastEntryGreenTime = 0;
static volatile uint32_t nextEntryGreenTime = 0;
static volatile uint16_t  lastEntryGreenIntensity = 0;
static volatile uint16_t  nextEntryGreenIntensity = 0;

static volatile uint32_t lastEntryBlueTime = 0;
static volatile uint32_t nextEntryBlueTime = 0;
static volatile uint16_t  lastEntryBlueIntensity = 0;
static volatile uint16_t  nextEntryBlueIntensity = 0;

static volatile uint32_t lastEntryWhiteTime = 0;
static volatile uint32_t nextEntryWhiteTime = 0;
static volatile uint16_t  lastEntryWhiteIntensity = 0;
static volatile uint16_t  nextEntryWhiteIntensity = 0;
//Das war der Cache

uint32_t getTimeMs(void){
	return timeMs;
}
void setTimeMs(uint32_t t){
	timeMs = t;
}

//Diese Methode bereitet die Outputports vor (Interface C12).
void Output_init(void){
	//PORTA ist komplett auf Input eingestellt für RTC und Buttons
	//SFIOR &= ~(1<<PUD);
	PORTA = (1<<PA3) | (1<<PA4) | (1<<PA5) | (1<<PA6) | (1<<PA7); //Pullupwiderstände für Buttons aktivieren
	DDRA  = 0;

	//PORTB ist nur für den Blaukanal zuständig, es gibt eine freie SPI-Schnittstelle
	PORTB = 0; //_BV(PINB7); zum aktivieren des Pullup-Widerstands
	DDRB  = (1<<PB3);// set port B Pin3 for output

	//PORTC ist das Interface für das Display, alles ist Output
	DDRC = 0xFF;

	/*PORTD PD0,PD1,PD4,PD5,PD7 sind im alternativen Modus
	 * PD6 (K1) und PD2 (PS_ON) sind Output
	 * PD3 (PWR_OK) ist input
	 */
	DDRD = (1<<PD2) | (1<<PD6) | (1<<PD4) | (1<<PD5) | (1<<PD7);
}

void Timer0_Init(void)
{
	TCCR0 = (1 << COM01) | (1 << WGM00) | (1 << WGM01) | (1 << CS02);

	/* 16-Bit Version                                                *
	 * Hier verwenden wir keinen Prescaler, d.h. der Counter läuft   *
	 * mit 16MHz. Bedeutet eine Wiederholungsrate von                *
	 * 16.000.000 / 65536 = 244Hz                                    */
	ICR1 = 0xFFFF; //Definiere einen festen TOP-Wert für PWM, in diesem Fall 16Bit.
	TCCR1A = (1 << COM1A1) | (0 << COM1A0) | (1 << COM1B1) | (0 << COM1B0) | (1 << WGM11);
	TCCR1B = (1 << WGM12) | (1 << WGM13) | (1 << CS10);

	/*
	 * 10Bit Version
	 */
	/*TCCR1A = (1 << COM1A1) | (1 << COM1B1) | (1<<WGM10) | (1<<WGM11);
	TCCR1B = (1<<CS11) | (1<<CS10) | (1 << WGM12);*/

	TCCR2 = (1 << COM21) | (1 << WGM20) | (1 << WGM21) | (1 << CS22);
}

void setCalculateFlag(void){
	calculateFlag = 1;
}

void cacheKeeper(void){
	if(!LOCK)
		LOCK=TRUE;
	else
		return;

	if(!(lastEntryRedTime <= timeMs && nextEntryRedTime >= timeMs
		 && lastEntryGreenTime <= timeMs && nextEntryGreenTime >= timeMs
		 && lastEntryBlueTime <= timeMs && nextEntryBlueTime >= timeMs
		 && lastEntryWhiteTime <= timeMs && nextEntryWhiteTime >= timeMs)){
			 //Cache muss erneuert werden
			 lastEntryRedTime = lastEntryGreenTime = lastEntryBlueTime = lastEntryWhiteTime = 0x00000000;
	    	 nextEntryRedTime = nextEntryGreenTime = nextEntryBlueTime = nextEntryWhiteTime = 0xFFFFFFFF;

			 dbEntry currentEntry;
			 currentEntry = getFirstEntry();
			 uint8_t run = TRUE;

			#ifdef DEBUG
			sendString("\nErneuere Cache");
			#endif

			 do{
			 	#ifdef DEBUG
			 	sendChar('.');
			 	#endif
				 if(currentEntry.channel==0){//0 bedeutet Rot
					 if(timeMs >= currentEntry.timestamp
					    && currentEntry.timestamp >= lastEntryRedTime){
						 /*Wenn der aktuelle Eintrag in der Vergangenheit liegt
						   UND nach dem letzten übernommenen Ereignis liegt
						   => dann erneuere den Cache-Eintrag*/
						 lastEntryRedTime = currentEntry.timestamp;
						 lastEntryRedIntensity = currentEntry.intensity;
					 }else if(timeMs <= currentEntry.timestamp && currentEntry.timestamp <= nextEntryRedTime){
						 /*Wenn der aktuelle Eintrag in der Zukunft liegt
						   UND vor dem aktuell gespeicherten Eintrag liegt
						   => dann erneuere den Cache-Eintrag*/
						 nextEntryRedTime = currentEntry.timestamp;
						 nextEntryRedIntensity = currentEntry.intensity;
					 }
				 }else if(currentEntry.channel==1){
					 if(timeMs >= currentEntry.timestamp
					   && lastEntryGreenTime <= currentEntry.timestamp){
						 lastEntryGreenTime = currentEntry.timestamp;
						 lastEntryGreenIntensity = currentEntry.intensity;
					 }else if(timeMs <= currentEntry.timestamp && currentEntry.timestamp <= nextEntryGreenTime){
						 nextEntryGreenTime = currentEntry.timestamp;
						 nextEntryGreenIntensity = currentEntry.intensity;
					 }
				 }else if(currentEntry.channel==2){
					 if(timeMs >= currentEntry.timestamp && lastEntryBlueTime <= currentEntry.timestamp){
						 lastEntryBlueTime = currentEntry.timestamp;
						 lastEntryBlueIntensity = currentEntry.intensity;
					 }else if(timeMs <= currentEntry.timestamp && currentEntry.timestamp <= nextEntryBlueTime){
						 nextEntryBlueTime = currentEntry.timestamp;
						 nextEntryBlueIntensity = currentEntry.intensity;
					 }
				 }else if(currentEntry.channel==3){
					 if(timeMs >= currentEntry.timestamp && lastEntryWhiteTime <= currentEntry.timestamp){
						 lastEntryWhiteTime = currentEntry.timestamp;
						 lastEntryWhiteIntensity = currentEntry.intensity;
					 }else if(timeMs <= currentEntry.timestamp && currentEntry.timestamp <= nextEntryWhiteTime){
						 nextEntryWhiteTime = currentEntry.timestamp;
						 nextEntryWhiteIntensity = currentEntry.intensity;
					 }
				 }
				 run = hasNextEntry();
				 if(run)
				 	currentEntry = getNextEntry();
			 }while(run);
			 /*
			 Jo ganz schlaue Sache: Wenn kein passender Eintrag gefunden wird bleibt der letzt gültige Eintrag
			 im Cache mit einem angepassten Zeitstempel. Das ist vorteilhaft für den "lastEntry" um Mitternacht zu überstehen.
			 Problem: was ist mit dem "nextEntry", wenn er nach Mitternacht liegt?
			 Ein Workaround wäre die Frühesten Einträge jedes Kanals noch einmal mit einem hohen,
			 nicht erreichbaren Zeitstempel (z.B. 0x05265C01) doppelt zu speichern.
			 */
			#ifdef DEBUG
			 	sendString("Cache Erneuert!\n");

			 	sendString("lRt: ");
			 	sendInteger(lastEntryRedTime);
			 	sendString(", lRi: ");
			 	sendInteger(lastEntryRedIntensity);
			 	sendString(", nRt: ");
			 	sendInteger(nextEntryRedTime);
			 	sendString(", nRi: ");
			 	sendInteger(nextEntryRedIntensity);
			 	sendChar('\n');

			 	sendString("lGt: ");
			 	sendInteger(lastEntryGreenTime);
			 	sendString(", lGi: ");
			 	sendInteger(lastEntryGreenIntensity);
			 	sendString(", nGt: ");
			 	sendInteger(nextEntryGreenTime);
			 	sendString(", nGi: ");
			 	sendInteger(nextEntryGreenIntensity);
			 	sendChar('\n');

			 	sendString("lBt: ");
			 	sendInteger(lastEntryBlueTime);
			 	sendString(", lBi: ");
			 	sendInteger(lastEntryBlueIntensity);
			 	sendString(", nBt: ");
			 	sendInteger(nextEntryBlueTime);
			 	sendString(", nBi: ");
			 	sendInteger(nextEntryBlueIntensity);
			 	sendChar('\n');

			 	sendString("lWt: ");
			 	sendInteger(lastEntryWhiteTime);
			 	sendString(", lWi: ");
			 	sendInteger(lastEntryWhiteIntensity);
			 	sendString(", nWt: ");
			 	sendInteger(nextEntryWhiteTime);
			 	sendString(", nWi: ");
			 	sendInteger(nextEntryWhiteIntensity);
			 	sendChar('\n');
			 #endif
	}
	LOCK = FALSE;
}

void calculateIntensities(void){
	float weight;
	uint32_t timeSpan;
	uint16_t oldval;

	//Berechne die Intensität von Rot

	oldval = intensityRed;

	timeSpan = nextEntryRedTime - lastEntryRedTime;

	weight = ((float)(timeMs - lastEntryRedTime) / (float)timeSpan);

	intensityRed = (uint8_t)(((float)(lastEntryRedIntensity)*(1-weight))
					+ ((float)(nextEntryRedIntensity)*weight));

	if(intensityRed == 0 && oldval > 0){
		TCCR2 &= ~(1 << COM21);
	}else if(intensityRed > 0 && oldval == 0){
		TCCR2 |= (1 << COM21);
	}

	OCR2 = intensityRed;

	//Berechne die Intensität von Grün

	oldval = intensityGreen;

	timeSpan = nextEntryGreenTime - lastEntryGreenTime;

	weight = ((float)(timeMs - lastEntryGreenTime) / (float)timeSpan);

	intensityGreen = (uint8_t)(((float)(lastEntryGreenIntensity)*(1-weight))
					+ ((float)(nextEntryGreenIntensity)*weight));

	if(intensityGreen == 0 && oldval > 0){
		TCCR0 &= ~(1 << COM01);
	}else if(intensityGreen > 0 && oldval == 0){
		TCCR0 |= (1 << COM01);
	}

	OCR0 = intensityGreen;

	//Berechne die Intensität von Blau

	oldval = intensityBlue;

	timeSpan = nextEntryBlueTime - lastEntryBlueTime;

	weight = ((float)(timeMs - lastEntryBlueTime) / (float)timeSpan);

	intensityBlue = (uint16_t)(((float)(lastEntryBlueIntensity<<8)*(1-weight))
					+ ((float)(nextEntryBlueIntensity<<8)*weight));

	if(intensityBlue == 0 && oldval > 0){
		TCCR1A &= ~(1 << COM1B1);
	}else if(intensityBlue > 0 && oldval == 0){
		TCCR1A |= (1 << COM1B1);
	}

	OCR1B = intensityBlue;

	//Berechne die Intensität von Weiß
	oldval = intensityWhite;

	timeSpan = nextEntryWhiteTime - lastEntryWhiteTime;

	weight = ((float)(timeMs - lastEntryWhiteTime) / (float)timeSpan);

	intensityWhite = (uint16_t)(((float)(lastEntryWhiteIntensity<<8)*(1-weight))
					+ ((float)(nextEntryWhiteIntensity<<8)*weight));

	if(intensityWhite == 0 && oldval > 0){
		TCCR1A &= ~(1 << COM1A1);
	}else if(intensityWhite > 0 && oldval == 0){
		TCCR1A |= (1 << COM1A1);
	}

	if(intensityWhite == 0
		&& isOnATX()){

		OCR1A = intensityWhite; //Zur Sicherheit

		//LEDs ausschalten
		TCCR0 &= ~(1 << COM01);
		TCCR1A &= ~((1 << COM1A1) | (1 << COM1B1));
		TCCR2 &= ~(1 << COM21);
		_delay_loop_1(0xFF);

		switchToVoltageRegulator();
		switchOffATX();

		//LEDs einschalten
		_delay_loop_2(0x8888);
		if(intensityGreen > 0){
			TCCR0 |= (1 << COM01);
		}
		if(intensityBlue > 0){
			TCCR1A |= (1 << COM1B1);
		}
		if(intensityRed > 0){
			TCCR2 |= (1 << COM21);
		}
	}else if(intensityWhite > 0 && !isOnATX()){
		//LEDs ausschalten
		TCCR0 &= ~(1 << COM01);
		TCCR1A &= ~((1 << COM1A1) | (1 << COM1B1));
		TCCR2 &= ~(1 << COM21);
		_delay_loop_1(0xFF);

		switchOnATX();
		switchToVoltageSupply();

		//LEDs einschalten
		_delay_loop_2(0xFFFF);
		if(intensityGreen > 0){
			TCCR0 |= (1 << COM01);
		}
		if(intensityBlue > 0){
			TCCR1A |= (1 << COM1B1);
		}
		if(intensityRed > 0){
			TCCR2 |= (1 << COM21);
		}
		TCCR1A |= (1 << COM1A1);
	}

	OCR1A = intensityWhite;

	lcd_gotoxy(0,0);
	lcd_putc('R');
	lcd_putu((intensityRed));
	lcd_puts("G");
	lcd_putu(intensityGreen);
	lcd_puts("B");
	lcd_putu(intensityBlue>>8);
	lcd_puts("W");
	lcd_putu((intensityWhite>>8));
	lcd_puts("        ");
}

//Unsere Main
int main (void)
{
	//Init PORTS
	switchOffATX();
	Output_init();

	//Init RTC
	RTC_init();

	//RS232 Initialisieren - wenn benötigt.
	USARTinit();
	//sendString("Prometheus erwacht.");

	//Start the Timer0
	Timer0_Init();

	//Ein und Ausgabe
	lcd_init(LCD_DISP_ON);
	//ADC_init();

	//Begrüßung
	lcd_BacklightOn();

	lcd_clrscr();
	lcd_puts("Prometheus-LMC");
	sendString(".\nPrometheus ist erwacht!\n");

	TIMSK = (1<<TOIE0);
}
