//***************************************************************************
//
//  File........: button.c
//
//  Author(s)...: ATMEL Norway
//
//  Target(s)...: ATmega169
//
//  Compiler....: AVR-GCC 3.3.1; avr-libc 1.0
//
//  Description.: AVR Butterfly button handling routines
//
//  Revisions...: 1.0
//
//  YYYYMMDD - VER. - COMMENT                                       - SIGN.
//
//  20030116 - 1.0  - Created                                       - KS
//  20031009          port to avr-gcc/avr-libc                      - M.Thomas
//  20060718		  stripped down									- U. Koeckemann
//
//***************************************************************************

#include "button.h"
#include "ADC.h"

// mt/v6 - keyclick (see main.c)
extern volatile char gKeyClickStatus;

char LOCK = FALSE;

// extern char gPowerSaveTimer;
//extern volatile uint8_t gPowerSaveTimer;

// mt  unsigned char gButtonTimeout = FALSE;
volatile unsigned char gButtonTimeout = FALSE;

//mtA
char KEY = NULL;
char KEY_VALID = FALSE;
//mtE

char CountdownTimerHandle;

/*****************************************************************************
*
*   Function name : Button_Init
*
*   Returns :       None
*
*   Parameters :    None
*
*   Purpose :       Initializes the five button pin
*
*****************************************************************************/
/*void Button_Init(void)
{
    // Init port pins
    PORTA |= PINA_MASK;  //Enable PullUps
    DDRA &= (~PINA_MASK); //Set Input
}*/

/*****************************************************************************
*
*   Function name : PinChangeInterrupt
*
*   Returns :       None
*
*   Parameters :    None
*
*   Purpose :       Check status on the joystick
*
*****************************************************************************/
void keyHandler(void)
{
    //char buttons;

    char key;

/*
    Read the buttons:

    Bit             7   6   5   4   3   2   1   0
    ---------------------------------------------
    PORTA           O   D   B   C   A
    =============================================
*/


    /*buttons = (~PINA) & PINA_MASK;

    // Output virtual keys
    if (!(buttons & (1<<BUTTON_A)))
        key = KEY_PLUS;
    else if (!(buttons & (1<<BUTTON_B)))
        key = KEY_MINUS;
    else if (!(buttons & (1<<BUTTON_C)))
        key = KEY_PREV;
    else if (!(buttons & (1<<BUTTON_D)))
        key = KEY_NEXT;
    else if (!(buttons & (1<<BUTTON_O))){
        key = KEY_ENTER;
	}
    else{
        key = KEY_NULL;
	}*/

	/*
	 * Workaround: durch den Einsatz des ADCs werden die fehlenden
	 * Pullup-Widerstände durch µC interne ersetzt.
	 */

	ADC_init();
	ADC_read(0);

	if (ADC_read(BUTTON_A) < 800){
        key = KEY_PLUS;
    }else if (ADC_read(BUTTON_B) < 800){
        key = KEY_MINUS;
    }else if (ADC_read(BUTTON_C) < 800){
        key = KEY_PREV;
    }else if (ADC_read(BUTTON_D) < 800){
        key = KEY_NEXT;
    }else if (ADC_read(BUTTON_O) < 800){
        key = KEY_ENTER;
	}else{
        key = KEY_NULL;
	}

	ADC_deactivate();


    if(key != KEY_NULL)
    {
		KEY = key;          // Store key in global key buffer
		KEY_VALID = TRUE;
		//LCD Zustand und Anzeige erneuern
		setBacklightAutoOff(3600);

		//if(!(TIMSK0&OCIE0A)){
//		refreshState();
			//refreshMenu();
		/*}else{
			refreshState();
		}*/
    }else{
    	KEY_VALID = FALSE;
	}
}



/*
 * Das LMC verzichtet auf den Luxus einer Interruptbasierten Buttonsteuerung
 * Die Buttons müssen aktiv in angemessenen Zeitabständen (z.B. 200ms)
 * abgefragt werden.
 *
 */



/*****************************************************************************
*
*   Function name : getkey
*
*   Returns :       The valid key
*
*   Parameters :    None
*
*   Purpose :       Get the valid key
*
*****************************************************************************/
char getkey(void)
{
    char k;

    //cli(); // mt: __disable_interrupt();
	if(!LOCK)
		LOCK = TRUE;
	else
		return KEY_NULL;


    if (KEY_VALID)              // Check for unread key in buffer
    {
        k = KEY;
        KEY_VALID = FALSE;
    }
    else
        k = KEY_NULL;           // No key stroke available

    //sei(); // mt: __enable_interrupt();
	LOCK = FALSE;

    return k;
}
