/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
 #include <avr/io.h>
 #include <stdlib.h>
 
 #include "database.h"
 static volatile uint8_t index = 0;

 uint8_t hasNextEntry(void){
	 if(index>=84)
		 return 0;
	 
	 uint16_t adr = (6*index);
	 return (eeprom_read_byte_32(&adr) != 0xFF);
 }
 
 uint8_t storeEntry(dbEntry newEntry){
	 uint16_t adr = 0;
	 uint8_t i = 0;
	 
	 while(eeprom_read_byte_32(&adr) != 0xFF){
		i++;
		adr = (6*i);
	 }
	 
	 if(i>=85)
		 return 0;
	 
	 eeprom_write_byte_32(&adr, (uint8_t)(newEntry.timestamp>>24));
	 adr++;
	 eeprom_write_byte_32(&adr, (uint8_t)(newEntry.timestamp>>16));
	 adr++;
	 eeprom_write_byte_32(&adr, (uint8_t)(newEntry.timestamp>>8));
	 adr++;
	 eeprom_write_byte_32(&adr, (uint8_t)newEntry.timestamp);
	 adr++;
	 eeprom_write_byte_32(&adr, (uint8_t)newEntry.channel);
	 adr++;
	 eeprom_write_byte_32(&adr, (uint8_t)newEntry.intensity);
	 
	 return i;
 }
 
 dbEntry getEntry(uint16_t idx){
	 uint16_t adr = (6*idx);
	 
	 dbEntry result;
	 	 
	 result.timestamp = ((uint32_t)eeprom_read_byte_32(&adr))<<24;
	 adr++;
	 result.timestamp |= ((uint32_t)eeprom_read_byte_32(&adr))<<16;
	 adr++;
	 result.timestamp |= ((uint32_t)eeprom_read_byte_32(&adr))<<8;
	 adr++;
	 result.timestamp |= eeprom_read_byte_32(&adr);
	 adr++;
	 result.channel = eeprom_read_byte_32(&adr);
	 adr++;
	 result.intensity = eeprom_read_byte_32(&adr);
	 
	 return result;
 }
 
 dbEntry getNextEntry(void){	 
	 index++;
	 
	 return getEntry(index);
 }
   
 dbEntry getFirstEntry(void){
	 index = 0;

	 return getEntry(index);
 }
 
 void formatDB(void){
	 uint16_t i;
	 for(i=0; i<512; i++){
		 eeprom_write_byte_32(&i, 0xFF);
	 }
 }
